LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;
USE work.my_package.all;

---------------------------------------------------
-- MY FIR filter
-- Computation of the coefficients
-- Version 2.0
-- 11 coefficients 
-- Author: Vebjrn Bystrm, NTNU 8.may 2008
---------------------------------------------------

PACKAGE my_functions is 

FUNCTION coeff_0 (signal c: in std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_1 (signal c : std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_2 (signal c : std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_3 (signal c : std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_4 (signal c : std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_5 (signal c : std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_6 (signal c : std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_7 (signal c : std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_8 (signal c : std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_9 (signal c : std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_10 (signal c : std_logic_vector) 
RETURN std_logic_vector;

END my_functions;

PACKAGE BODY my_functions is 


--------------  COEFF_0 ---------------------------
	FUNCTION coeff_0 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
		
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);		
			
	BEGIN
		shift1 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7 downto 0)&'0');
        
		return shift1;
	END coeff_0;
----------------------------------------------------

--------------  COEFF_1 ----------------------------
	FUNCTION coeff_1 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
		
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 :std_logic_vector (23 DOWNTO 0);
		
	BEGIN
		shift1 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7 downto 0)&"0000");
		shift2 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7)&c(7)&c(7 downto 0)&'0');
        		
		return (shift1-shift2);
				
	END coeff_1;
----------------------------------------------------


--------------  COEFF_2 ----------------------------
	FUNCTION coeff_2 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
				
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift3 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift4 :std_logic_vector (23 DOWNTO 0);
		
	BEGIN
		shift1 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7 downto 0)&"000000");
		shift2 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7 downto 0)&"0000");
		shift3 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7)&c(7 downto 0)&"000");
		shift4 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7)&c(7)&c(7)&c(7)&c(7 downto 0));
        
		return ('0' - shift1 + shift2 + shift3 - shift4);
				
	END coeff_2;
----------------------------------------------------

--------------  COEFF_3 ----------------------------
	FUNCTION coeff_3 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
		
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift3 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift4 :std_logic_vector (23 DOWNTO 0);
	
	BEGIN
		shift1 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7 downto 0)&"000000");
		shift2 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7 downto 0)&"0000");
		shift3 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7)&c(7 downto 0)&"000");
		shift4 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7)&c(7)&c(7)&c(7)&c(7 downto 0));
        
		return ('0' - shift1 + shift2 + shift3 - shift4);
	
	END coeff_3;
----------------------------------------------------
	

--------------  COEFF_4 ----------------------------	
	FUNCTION coeff_4 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
		
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift3 :std_logic_vector (23 DOWNTO 0);
			
	BEGIN
		
		shift1 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7 downto 0)&"00000000");
		shift2 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7 downto 0)&"00000");
		shift3 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7)&c(7 downto 0)&"000");
		        
		return (shift1 + shift2 + shift3);
	
	END coeff_4;
----------------------------------------------------

--------------  COEFF_5 ----------------------------
	
	FUNCTION coeff_5 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
		
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift3 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift4 :std_logic_vector (23 DOWNTO 0);
	
	BEGIN
		shift1 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7 downto 0)&"000000000");
		shift2 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7 downto 0)&"000000");
		shift3 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7)&c(7 downto 0)&"000");
		shift4 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7)&c(7)&c(7)&c(7 downto 0)&"0");
        
		return (shift1 + shift2 - shift3 - shift4);
	
	END coeff_5;
-----------------------------------------------------

--------------  COEFF_6 -----------------------------
	FUNCTION coeff_6 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
		
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift3 :std_logic_vector (23 DOWNTO 0);
		
	
	BEGIN
		shift1 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7 downto 0)&"00000000");
		shift2 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7 downto 0)&"00000");
		shift3 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7)&c(7)&c(7 downto 0)&"000");
		        
		return (shift1 + shift2 + shift3);
	
	END coeff_6;
-----------------------------------------------------
--------------  COEFF_7 -----------------------------
	FUNCTION coeff_7 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
		
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift3 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift4 :std_logic_vector (23 DOWNTO 0);
	
	BEGIN
		shift1 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7 downto 0)&"000000");
		shift2 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7 downto 0)&"0000");
		shift3 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7)&c(7 downto 0)&"000");
		shift4 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7)&c(7)&c(7)&c(7)&c(7 downto 0));
        
		return ('0' - shift1 + shift2 + shift3 - shift4);
	
	END coeff_7;
----------------------------------------------------


--------------  COEFF_8 ----------------------------	
	FUNCTION coeff_8 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
		
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift3 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift4 :std_logic_vector (23 DOWNTO 0);
	
	BEGIN
		shift1 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7 downto 0)&"000000");
		shift2 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7 downto 0)&"0000");
		shift3 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7)&c(7 downto 0)&"000");
		shift4 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7)&c(7)&c(7)&c(7)&c(7 downto 0));
        
		return ('0' - shift1 + shift2 + shift3 - shift4);
	
	END coeff_8;
----------------------------------------------------

--------------  COEFF_9 ----------------------------
	FUNCTION coeff_9 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
		
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 :std_logic_vector (23 DOWNTO 0);
			
	BEGIN
		shift1 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7 downto 0)&"0000");
		shift2 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7)&c(7)&c(7 downto 0)&'0');
		
        
		return (shift1 - shift2);
	
	END coeff_9;
----------------------------------------------------

--------------  COEFF_10 ---------------------------
	FUNCTION coeff_10 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
		
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
			
	BEGIN
		shift1 := (c(7)&c(7)&c(7)&c(7)&
		c(7)&c(7)&c(7)&c(7)&c(7)&c(7)&c(7)
		&c(7)&c(7)&c(7)&c(7)&c(7 downto 0)&'0');
		
        
		return (shift1);
	
	END coeff_10;
----------------------------------------------------


END my_functions;