LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;
USE work.my_package.all;

--------------------------------------------------
-- MY FIR filter
-- Version 4.0
-- 11 coefficients 
-- Binary representation of the coefficients
-- Author: Vebjrn Bystrm, NTNU 22.may 2008
---------------------------------------------------

PACKAGE my_functions is 

FUNCTION coeff_0 (signal c: in std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_1 (signal c : std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_2 (signal c : std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_3 (signal c : std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_4 (signal c : std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_5 (signal c : std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_6 (signal c : std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_7 (signal c : std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_8 (signal c : std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_9 (signal c : std_logic_vector) 
RETURN std_logic_vector;
FUNCTION coeff_10 (signal c : std_logic_vector) 
RETURN std_logic_vector;

END my_functions;

PACKAGE BODY my_functions is 


--------------  COEFF_0 -----------------------------------------------	
	FUNCTION coeff_0 (signal c: in std_logic_vector)
 RETURN std_logic_vector IS
		
		VARIABLE temp : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 : std_logic_vector (23 DOWNTO 0);
			
	BEGIN
		temp := c & "0000000000000000";
		temp := (temp sra 16);
		shift1 := (temp sll 3);
		shift2 := (temp sll 4);
        
		return shift1 + shift2;
	END coeff_0;
-----------------------------------------------------------------------

--------------  COEFF_1 -----------------------------------------------	
	FUNCTION coeff_1 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
		
		VARIABLE temp : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 :std_logic_vector (23 DOWNTO 0);
				
	BEGIN
		temp := c & "0000000000000000";
		temp := (temp sra 16);
		shift1 := (temp sll 3);
		shift2 := (temp sll 7);
		        		
		return (temp + shift1 + shift2);
				
	END coeff_1;
-----------------------------------------------------------------------

--------------  COEFF_2 -----------------------------------------------	
	FUNCTION coeff_2 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
				
		VARIABLE temp : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift3 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift4 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift5 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift6 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift7 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift8 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift9 :std_logic_vector (23 DOWNTO 0);
		
	BEGIN
		temp := c & "0000000000000000";
		temp := (temp sra 16);
		shift1 := (temp sll 2);
		shift2 := (temp sll 6);
		shift3 := (temp sll 9);
		shift4 := (temp sll 10);
		shift5 := (temp sll 11);
		shift6 := (temp sll 12);
		shift7 := (temp sll 13);
		shift8 := (temp sll 14);
        shift9 := (temp sll 15);
		
		return (temp + shift1 + shift2 + shift3 + 
		shift4 + shift5 + shift6 + shift7 + shift8 
		+ shift9);
				
	END coeff_2;
-----------------------------------------------------------------------

--------------  COEFF_3 -----------------------------------------------	
	FUNCTION coeff_3 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
		
		VARIABLE temp : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift3 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift4 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift5 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift6 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift7 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift8 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift9 :std_logic_vector (23 DOWNTO 0);
	
	BEGIN
		temp := c & "0000000000000000";
		temp := (temp sra 16);
		shift1 := (temp sll 2);
		shift2 := (temp sll 6);
		shift3 := (temp sll 9);
		shift4 := (temp sll 10);
		shift5 := (temp sll 11);
		shift6 := (temp sll 12);
		shift7 := (temp sll 13);
		shift8 := (temp sll 14);
        shift9 := (temp sll 15);
		
		return (shift1 + shift2 + shift3 
		+ shift4 + shift5 + shift6 + shift7 
		+ shift8 + shift9);
	
	END coeff_3;
-----------------------------------------------------------------------
	

--------------  COEFF_4 -----------------------------------------------	
	FUNCTION coeff_4 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
		
		VARIABLE temp : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift3 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift4 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift5 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift6 :std_logic_vector (23 DOWNTO 0);
	
	BEGIN
		temp := c & "0000000000000000";
		temp := (temp sra 16);
		shift1 := (temp srl 1);
		shift2 := (temp srl 4);
		shift3 := (temp srl 6);
		shift4 := (temp srl 8);
		shift5 := (temp srl 9);
		shift6 := (temp srl 11);
		        
		return (shift1 + shift2 + shift3 
		+ shift4 + shift5 + shift6);
	
	END coeff_4;
-----------------------------------------------------------------------

--------------  COEFF_5 -----------------------------------------------	
	FUNCTION coeff_5 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
		
		VARIABLE temp : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift3 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift4 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift5 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift6 :std_logic_vector (23 DOWNTO 0);
	
	BEGIN
		temp := c & "0000000000000000";
		temp := (temp sra 16);
		shift1 := (temp srl 3);
		shift2 := (temp srl 6);
		shift3 := (temp srl 7);
		shift4 := (temp srl 8);
		shift5 := (temp srl 10);
		shift6 := (temp srl 12);
		        
		return (shift1 + shift2 + shift3 
		+ shift4 + shift5 + shift6);
	
	END coeff_5;
-----------------------------------------------------------------------

--------------  COEFF_6 -----------------------------------------------	
	FUNCTION coeff_6 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
		
		VARIABLE temp : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift3 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift4 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift5 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift6 :std_logic_vector (23 DOWNTO 0);
	
	BEGIN
		temp := c & "0000000000000000";
		temp := (temp sra 16);
		shift1 := (temp srl 1);
		shift2 := (temp srl 4);
		shift3 := (temp srl 6);
		shift4 := (temp srl 8);
		shift5 := (temp srl 9);
		shift6 := (temp srl 11);
		        
		return (shift1 + shift2 + shift3 
		+ shift4 + shift5 + shift6);
	
	END coeff_6;
-----------------------------------------------------------------------

--------------  COEFF_7 -----------------------------------------------	
	FUNCTION coeff_7 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
		
		VARIABLE temp : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift3 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift4 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift5 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift6 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift7 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift8 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift9 :std_logic_vector (23 DOWNTO 0);
	
	BEGIN
		temp := c & "0000000000000000";
		temp := (temp sra 16);
		shift1 := (temp sll 2);
		shift2 := (temp sll 6);
		shift3 := (temp sll 9);
		shift4 := (temp sll 10);
		shift5 := (temp sll 11);
		shift6 := (temp sll 12);
		shift7 := (temp sll 13);
		shift8 := (temp sll 14);
        shift9 := (temp sll 15);
		
		return (shift1 + shift2 + shift3 
		+ shift4 + shift5 + shift6 + shift7
		+ shift8 + shift9);
	
	END coeff_7;
-----------------------------------------------------------------------


--------------  COEFF_8 -----------------------------------------------	
	FUNCTION coeff_8 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
		
		VARIABLE temp : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift3 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift4 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift5 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift6 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift7 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift8 :std_logic_vector (23 DOWNTO 0);
		VARIABLE shift9 :std_logic_vector (23 DOWNTO 0);
		
	BEGIN
		temp := c & "0000000000000000";
		temp := (temp sra 16);
		shift1 := (temp sll 2);
		shift2 := (temp sll 6);
		shift3 := (temp sll 9);
		shift4 := (temp sll 10);
		shift5 := (temp sll 11);
		shift6 := (temp sll 12);
		shift7 := (temp sll 13);
		shift8 := (temp sll 14);
        shift9 := (temp sll 15);
		
		return (temp + shift1 + shift2 + shift3 
		+ shift4 + shift5 + shift6 + shift7 
		+ shift8 + shift9);
	
	END coeff_8;
-----------------------------------------------------------------------

--------------  COEFF_9 -----------------------------------------------	
	FUNCTION coeff_9 (signal c: in std_logic_vector) 
	RETURN std_logic_vector IS
		
		VARIABLE temp : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 :std_logic_vector (23 DOWNTO 0);
				
	BEGIN
		temp := c & "0000000000000000";
		temp := (temp sra 16);
		shift1 := (temp sll 3);
		shift2 := (temp sll 7);
		        		
		return (temp + shift1 + shift2);
	
	END coeff_9;
-----------------------------------------------------------------------

--------------  COEFF_10 -----------------------------------------------	
	FUNCTION coeff_10 (signal c: in std_logic_vector)
 RETURN std_logic_vector IS
		
		VARIABLE temp : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift1 : std_logic_vector (23 DOWNTO 0);
		VARIABLE shift2 : std_logic_vector (23 DOWNTO 0);
			
	BEGIN
		temp := c & "0000000000000000";
		temp := (temp sra 16);
		shift1 := (temp sll 3);
		shift2 := (temp sll 4);
        
		return shift1 + shift2;
	
	END coeff_10;
-----------------------------------------------------------------------

END my_functions;